-- Carenado Baron 58
-- Updated for LINDA 2.7
-- May 2016
-- V 1.1

-- Adapted from FSX Default module

-- ## Autopilot controls #####################################

function B58_AP_MASTER_toggle ()
	_AP_MASTER ()
end

function B58_AP_MASTER_on ()
	_AUTOPILOT_ON ()
end

function B58_AP_MASTER_off ()
	_AUTOPILOT_OFF ()
end

function B58_AP_NAV_hold ()
	_AP_NAV1_HOLD ()
end

function B58_AP_ATT_hold ()
	_AP_ATT_HOLD ()
end

function B58_AP_LOC_hold ()
	_AP_LOC_HOLD ()
end

function B58_AP_APR_hold ()
	_AP_APR_HOLD ()
end

function B58_AP_VS_hold ()
	_AP_VS_HOLD ()
end

function B58_AP_PANEL_VS_hold ()
	_AP_PANEL_VS_HOLD ()
end

function B58_AP_ALT_hold ()
	_AP_ALT_HOLD ()
end

function B58_AP_PANEL_ALT_hold ()
	_AP_PANEL_ALTITUDE_HOLD ()
end

function B58_AP_BC_hold ()
	_AP_BC_HOLD ()
end

function B58_AP_HDG_hold ()
	_AP_HDG_HOLD ()
end

function B58_AP_PANEL_HDG_hold ()
	_AP_PANEL_HEADING_HOLD ()
end

function B58_AP_HDG_BUG_align ()
	val = ipc.readDBL(0x2B00)
	ipc.writeUW("07cc", val/360*65536)
	DspHDG(val)
end

function B58_AP_FD_on ()
	ipc.writeSB("2EE0", 1)
end

function B58_AP_FD_off ()
	ipc.writeSB("2EE0", 0)
end

function B58_AP_TOGA_on ()
	ipc.writeSB("080C", 1)
end

function B58_AP_TOGA_off ()
	ipc.writeSB("080C", 0)
end

-- ## AP Knob functions

-- $$ heading knob

function B58_HDG_plus (a, b, c)
	if c == nil then
		_HEADING_BUG_INC ()
		return
	end
	sync_hdg = round(c / 360 * 65536)
	ipc.writeUW(0x07CC, sync_hdg)
	DspHDG(c)
    ipc.sleep(20)
end

function B58_HDG_plusfast (a, b, c)
	if c == nil then
		for i = 1, fast_hdg do
			_HEADING_BUG_INC ()
		end
		return
	end
	sync_hdg = round(c / 360 * 65536)
	ipc.writeUW(0x07CC, sync_hdg)
	DspHDG(c)
    ipc.sleep(20)
end

function B58_HDG_minus (a, b, c)
	if c == nil then
		_HEADING_BUG_DEC ()
		return
	end
	sync_hdg = round(c / 360 * 65536)
	ipc.writeUW(0x07CC, sync_hdg)
	DspHDG(c)
    ipc.sleep(20)
end

function B58_HDG_minusfast (a, b, c)
	if c == nil then
		for i = 1, fast_hdg do
			_HEADING_BUG_DEC ()
		end
		return
	end
	sync_hdg = round(c / 360 * 65536)
	ipc.writeUW(0x07CC, sync_hdg)
	DspHDG(c)
    ipc.sleep(20)
end

function B58_HDG_show ()
    Dsp3("HDG\\")
    HDG = round(ipc.readUW(0x07E2) * 360 / 65536)
	DspHDG(HDG)
	ipc.sleep(40)
end

-- $$ Altitude knob

function B58_ALT_plus (a, b, c)
	if c == nil then
		_AP_ALT_VAR_INC ()
		return
	end
    sync_alt = c*100/3.28084*65536
	ipc.writeUD(0x07D4, sync_alt)
	DspALT(c)
    ipc.sleep(20)
end

function B58_ALT_plusfast (a, b, c)
	if c == nil then
		for i = 1, fast_alt do
			_AP_ALT_VAR_INC ()
		end
		return
	end
    sync_alt = c*100/3.28084*65536
	ipc.writeUD(0x07D4, sync_alt)
	DspALT(c)
    ipc.sleep(20)
end

function B58_ALT_minus (a, b, c)
	if c == nil then
		_AP_ALT_VAR_DEC ()
		return
	end
    sync_alt = c*100/3.28084*65536
	ipc.writeUD(0x07D4, sync_alt)
	DspALT(c)
    ipc.sleep(20)
end

function B58_ALT_minusfast (a, b, c)
	if c == nil then
		for i = 1, fast_alt do
			_AP_ALT_VAR_DEC ()
		end
		return
	end
    sync_alt = c*100/3.28084*65536
    ipc.writeUD(0x07D4, sync_alt)
	DspALT(c)
    ipc.sleep(20)
end

-- $$ Vertical Velocity knob

function B58_VVS_plus (a, b, c)
    if _MCP1() then
    	ipc.writeUW(0x07F2, ipc.readUW(0x07F2) + 100)
    	VVS_show ()
    else
        ipc.writeUW(0x07F2, c * 100)
    end
    DspVVS(c)
    ipc.sleep(20)
end

function B58_VVS_minus (a, b, c)
    if _MCP1() then
        ipc.writeUW(0x07F2, ipc.readUW(0x07F2) - 100)
        VVS_show ()
    else
        ipc.writeUW(0x07F2, c * 100)
    end
    DspVVS(c)
    ipc.sleep(20)
end

function B58_VVS_show ()
	local buffer = ipc.readUW(0x07F2)
	if buffer > 16384 then
		buffer = 65536 - buffer
		line2 = string.format("-%03d", buffer/10)
	else
		line2 = string.format("+%03d", buffer/10)
	end
	if buffer == 0 then line2 = "00" end
	DspShow("VS", line2)
end

-- $$ Speed knob

function B58_SPD_plus (a, b, c)
	if c == nil then
		_AP_SPD_VAR_INC ()
		return
	end
	ipc.writeUW(0x07E2, c)
    sync_spd = c
	DspSPD(c)
    ipc.sleep(20)
end

function B58_SPD_plusfast (a, b, c)
	if c == nil then
		for i = 1, fast_spd do
			_AP_SPD_VAR_INC ()
		end
		return
	end
	ipc.writeUW(0x07E2, c)
    sync_spd = c
	DspSPD(c)
    ipc.sleep(20)
end

function B58_SPD_minus (a, b, c)
	if c == nil then
		_AP_SPD_VAR_DEC ()
		return
	end
	ipc.writeUW(0x07E2, c)
    sync_spd = c
	DspSPD(c)
    ipc.sleep(20)
end

function B58_SPD_minusfast (a, b, c)
	if c == nil then
		for i = 1, fast_spd do
			_AP_SPD_VAR_DEC ()
		end
		return
	end
	ipc.writeUW(0x07E2, c)
    sync_spd = c
	DspSPD(c)
    ipc.sleep(20)
end

function B58_SPD_show ()
    Dsp0("\\\\\\S")
    Dsp1("PD\\\\")
    SPD = ipc.readUW(0x07E2)
	DspSPD(SPD)
	ipc.sleep(40)
end

-- $$ Course knob
function B58_CRS_plus ()
	crs_hdg = crs_hdg + 1
	if crs_hdg > 359 then crs_hdg = 0 end
	CRS_show ()
end

function B58_CRS_plusfast ()
	crs_hdg = crs_hdg + fast_crs
	if crs_hdg > 359 then crs_hdg = 360 - crs_hdg end
	CRS_show ()
end

function B58_CRS_minus ()
	crs_hdg = crs_hdg - 1
	if crs_hdg < 0 then crs_hdg = 359 end
	CRS_show ()
end

function B58_CRS_minusfast ()
	crs_hdg = crs_hdg - fast_crs
	if crs_hdg < 0 then crs_hdg = 360 + crs_hdg end
	CRS_show ()
end

function B58_CRS_show ()
    if crs_open == 1 then
        -- OBS1
        ipc.writeUD(0x0C4E, crs_hdg)
        sync_crs = crs_hdg
    elseif crs_open == 2 then
        -- OBS2
        ipc.writeUD(0x0C5E, crs_hdg)
        sync_crs2 = crs_hdg
    elseif crs_open == 3 then
        -- ADF1
        ipc.writeUD(0x0C6C, crs_hdg)
        sync_adf = crs_hdg
    end
	if ipc.get("APlock") ~= 1 then
        if SPD_CRS_replace then
            DspCRS(crs_hdg, crs_open)
        else
            DspShow("CRS", DspNum(crs_hdg))
        end
    end
end

function B58_CRS_display_toggle ()
    crs_open = crs_open + 1
    if crs_open > 3 then crs_open = 1 end
    -- change label
    DspSPD2CRS ()
end

-- ## DME Selection

function B58_DME_off ()
    if ipc.readLvar('DMEPOS') ~= 0 then
        ipc.writeLvar('DMEPOS', 0)
    end
    DspShow('DME', 'off')
end

function B58_DME_n1 ()
    if ipc.readLvar('DMEPOS') ~= 1 then
        ipc.writeLvar('DMEPOS', 1)
    end
    DspShow('DME', 'n1')
end

function B58_DME_n2 ()
    if ipc.readLvar('DMEPOS') ~= 2 then
        ipc.writeLvar('DMEPOS', 2)
    end
    DspShow('DME', 'n2')
end

function B58_DME_inc ()
local pos = ipc.readLvar('DMEPOS')
    if pos < 2 then
        ipc.writeLvar('DMEPOS', pos + 1)
        DspShow('DME', 'n' .. (pos + 1))
    end
end

function B58_DME_dec ()
local pos = ipc.readLvar('DMEPOS')
    if pos > 0 then
        ipc.writeLvar('DMEPOS', pos - 1)
    end
    if pos - 1 == 0 then
        DspShow('DME', 'off')
    else
        DspShow('DME', 'n' .. (pos - 1))
    end
end

function B58_DME_cycle ()
local pos = ipc.readLvar('DMEPOS')
    if pos < 2 then
        B58_DME_inc ()
    else
        B58_DME_off ()
    end
end

-- ## GPS functions
-- %% only operates pwr button but not GNS 430

function B58_GPS_PWR_on ()
    if ipc.readLvar('offon4') ~= 1 then
        ipc.writeLvar('offon4', 1)
    end
end

function B58_GPS_PWR_off ()
    if ipc.readLvar('offon4') ~= 0 then
        ipc.writeLvar('offon4', 0)
    end
end

-- ## Radios functions  #####################################

function isAvionicsOn ()
	return ipc.readUB(0x2E80) == 1
end

function Radios_DME_AUDIO_toggle ()
    if logic.And(ipc.readUB(0x3122), 2) ~= 2 then
        if _MCP1 () then
            DspShow("DME","AUD")
        else
            if RADIOS_MODE == 4 then -- DME display is open
                DspRadioIdent_on ()
            else
                DspShow("DME","AUD","DME","Ident On")
            end
        end
    	ipc.control(65839)
	else
        if _MCP1 () then
            DspShow("DME","off")
        else
            if RADIOS_MODE == 4 then -- DME display is open
                DspRadioIdent_off ()
            else
                DspShow("DME","aud","  DME","IdentOff")
            end
        end
    	ipc.control(65834)
	end
end

function Radios_ADF_AUDIO_toggle ()
	_logg("ADF_AUDIO_toggle adf"..adf_sel)
	if adf_sel == 1 then
		if _t("adf1") then
            if _MCP1 () then
                DspShow("ADF1","aud")
            else
                DspRadioIdent_on ()
            end
			ipc.control(65841)
		else
            if _MCP1 () then
                DspShow("ADF1","off")
            else
                DspRadioIdent_off ()
            end
			ipc.control(65836)
		end
	else
		if _t("adf2") then
            if _MCP1 () then
                DspShow("ADF2","aud")
            else
                DspRadioIdent_on ()
            end
			ipc.control(66558)
		else
            if _MCP1 () then
                DspShow("ADF2","off")
            else
                DspRadioIdent_off ()
            end
			ipc.control(66557)
		end
	end
end

function Radios_NAV_AUDIO_toggle ()
	_logg("NAV_AUDIO_toggle nav"..nav_sel)
	if nav_sel == 1 then
		if _t("nav1") then
            if _MCP1 () then
                DspShow("NAV1","aud")
            else
                DspRadioIdent_on ()
            end
			ipc.control(65837)
		else
            if _MCP1 () then
                DspShow("NAV1","off")
            else
                DspRadioIdent_off ()
            end
			ipc.control(65832)
		end
	else
		if _t("nav2") then
            if _MCP1 () then
                DspShow("NAV2","aud")
            else
                DspRadioIdent_on ()
            end
			ipc.control(65838)
		else
            if _MCP1 () then
                DspShow("NAV2","off")
            else
                DspRadioIdent_off ()
            end
			ipc.control(65833)
		end
	end
end

function Radios_COM_AUDIO_toggle ()
	if _t("camaud") then
		ipc.control(66464)
        if _MCP1 () then
    		DspShow("COM2","aud")
        else
            if RADIOS_SUBMODE == 1 then
                DspRadioIdent_off ()
            else
                DspRadioIdent_on ()
            end
        end
	else
		ipc.control(66463)
        if _MCP1 () then
    		DspShow("COM1","aud")
        else
            if RADIOS_SUBMODE == 2 then
                DspRadioIdent_off ()
            else
                DspRadioIdent_on ()
            end
        end
	end
end

function Radios_OMI_AUDIO_toggle ()
    -- _MARKER_SOUND_TOGGLE ()
	local n = ipc.readUB("3122")
	n = logic.Xor(n, 4)
	ipc.writeUB("3122", n)
    _logg(tonumber(logic.And(n, 4)))
    if logic.And(n, 4) == 4 then
        DspShow ("MKR", "on")
    else
        DspShow ("MKR", "off")
    end
end

-- ## Altimeter baro functions  ###############################

function B58_Alt_BARO_STD_toggle ()
	if baro_mode ~= 10 then
		baro_cur = ipc.readUW(0x330)
    	ipc.control(66846)
		DspShow("Baro", "Std")
		baro_mode = 10
	else
    	ipc.control(65584)
		baro_cur = ipc.readUW(0x330)
		-- ipc.writeUW(0x330, baro_cur)
		baro_cur = round(((baro_cur * 2992) / (1013.2 * 16)) + 0.5)
		DspShow("Baro", tostring(baro_cur))
		baro_mode = 0
	end
end

function B58_Alt_BARO_MODE_hPa ()
	baro_mode = 1
	Altimeter_BARO_show ()
    ipc.writeLvar("hPa Button", 1)
end

function B58_Alt_BARO_MODE_inHg ()
	baro_mode = 0
	Altimeter_BARO_show ()
    ipc.writeLvar("inHg Button", 0)
end

function B58_Alt_BARO_MODE_mmHg ()
	baro_mode = 2
	Altimeter_BARO_show ()
    ipc.writeLvar("mmHg Button", 0)
end

function B58_Alt_BARO_MODE_toggle ()
	baro_mode = baro_mode + 1
    if baro_mode > 2 then baro_mode = 0 end
    if baro_mode == 0 then
		Altimeter_BARO_MODE_inHg ()
	elseif baro_mode == 1  then
		Altimeter_BARO_MODE_hPa ()
	else
		Altimeter_BARO_MODE_mmHg ()
	end
end

function B58_Alt_BARO_plus ()
	if not baro_std then
		baro = ipc.readUW(0x330)
		baro = baro + 4
		ipc.writeUW(0x330, baro)
	end
	Altimeter_BARO_show ()
end

function B58_Alt_BARO_plusfast ()
	if not baro_std then
		baro = ipc.readUW(0x330)
		baro = baro + fast_baro
		ipc.writeUW(0x330, baro)
	end
	Altimeter_BARO_show ()
end

function B58_Alt_BARO_minus ()
	if not baro_std then
		baro = ipc.readUW(0x330)
		baro = baro - 4
		ipc.writeUW(0x330, baro)
	end
	Altimeter_BARO_show ()
end

function B58_Alt_BARO_minusfast ()
	if not baro_std then
		baro = ipc.readUW(0x330)
		baro = baro - fast_baro
		ipc.writeUW(0x330, baro)
	end
	Altimeter_BARO_show ()
end

function B58_Alt_BARO_show ()
	local baro = ipc.readUW(0x330)
	-- hPa mode
	if baro_mode == 0 then
		l1 = "inHg"
		baro = round(((baro * 2992) / (1013.2 * 16)) + 0.5)
	-- mbar mode
	elseif baro_mode == 1 then
		l1 = " hPa"
		baro = baro / 16
	-- mm/m mode
	elseif baro_mode == 2 then
		l1 = "mmHg"
		baro = ((baro * 760) / (1013.2 * 16)) + 0.5
	else
		DspShow("Baro", "Std")
		return
	end
	DspShow(l1, string.format("%4d", baro))
end

-- ## Lights / Signs ###################################

-- $$ Cabin Lights

function B58_LIGHTS_CABIN_on ()
    B58_LIGHTS_FLOOD_on()
    B58_LIGHTS_PANEL_on()
end

function B58_LIGHTS_CABIN_off ()
    B58_LIGHTS_FLOOD_on()
    B58_LIGHTS_PANEL_off()
end

function B58_LIGHTS_CABIN_toggle ()
    B58_LIGHTS_FLOOD_toggle()
    B58_LIGHTS_PANEL_toggle()
end

-- $$ Flood Lights

function B58_LIGHTS_FLOOD_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 512)
    ipc.writeUW("0D0C", n)
	DspShow ("FLOD", "on")
end

function B58_LIGHTS_FLOOD_off ()
	ipc.clearbitsUW("0D0C", 512)
	DspShow ("FLOD", "off")
end

function B58_LIGHTS_FLOOD_toggle ()
    if logic.And(ipc.readUW("0D0C"), 512) == 512 then
		Lights_CABIN_off ()
	else
		Lights_CABIN_on ()
	end
end

-- $$ Panel Lights

function B58_LIGHTS_PANEL_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 32)
    ipc.writeUW("0D0C", n)
	DspShow ("PANL", "on")
end

function B58_LIGHTS_PANEL_off ()
    ipc.clearbitsUW("0D0C", 32)
	DspShow ("PANL", "off ")
end

function B58_LIGHTS_PANEL_toggle ()
    if logic.And(ipc.readUW("0D0C"), 32) == 32 then
		Lights_PANEL_off ()
	else
		Lights_PANEL_on ()
	end
end

function B58_LIGHTS_BEACON_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 2)
    ipc.writeUW("0D0C", n)
	DspShow ("BCN", "on")
end

function B58_LIGHTS_BEACON_off ()
	ipc.clearbitsUW("0D0C", 2)
	DspShow ("BCN", "off")
end

function B58_LIGHTS_BEACON_toggle ()
    if logic.And(ipc.readUW("0D0C"), 2) == 2 then
		Lights_BEACON_off ()
	else
		Lights_BEACON_on ()
	end
end

function B58_LIGHTS_STROBE_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 16)
    ipc.writeUW("0D0C", n)
	DspShow ("STRB", "on")
end

function B58_LIGHTS_STROBE_off ()
	ipc.clearbitsUW("0D0C", 16)
	DspShow ("STRB", "off")
end

function B58_LIGHTS_STROBE_toggle ()
    if logic.And(ipc.readUW("0D0C"), 16) == 16 then
		Lights_STROBE_off ()
	else
		Lights_STROBE_on ()
	end
end

function B58_LIGHTS_NAV_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 1)
    ipc.writeUW("0D0C", n)
	DspShow ("NAV", "on")
end

function B58_LIGHTS_NAV_off ()
	ipc.clearbitsUW("0D0C", 1)
    DspShow ("NAV", "off")
end

function B58_LIGHTS_NAV_toggle ()
    if logic.And(ipc.readUW("0D0C"), 1) == 1 then
		Lights_NAV_off ()
	else
		Lights_NAV_on ()
	end
end

function B58_LIGHTS_LANDING_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 4)
    ipc.writeUW("0D0C", n)
	DspShow ("LNDG", "on")
end

function B58_LIGHTS_LANDING_off ()
	ipc.clearbitsUW("0D0C", 4)
	DspShow ("LNDG", "off")
end

function B58_LIGHTS_LANDING_toggle ()
    if logic.And(ipc.readUW("0D0C"), 4) == 4 then
		Lights_LANDING_off ()
	else
		Lights_LANDING_on ()
	end
end

function B58_LIGHTS_TAXI_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 8)
    ipc.writeUW("0D0C", n)
	DspShow ("TAXI", "on")
end

function B58_LIGHTS_TAXI_off ()
	ipc.clearbitsUW("0D0C", 8)
	DspShow ("TAXI", "off")
end

function B58_LIGHTS_TAXI_toggle ()
    if logic.And(ipc.readUW("0D0C"), 8) == 8 then
		Lights_TAXI_off ()
	else
		Lights_TAXI_on ()
	end
end

function B58_LIGHTS_RECOGN_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 64)
    ipc.writeUW("0D0C", n)
	DspShow ("RCGN", "on")
end

function B58_LIGHTS_RECOGN_off ()
	ipc.clearbitsUW("0D0C", 64)
	DspShow ("RCGN", "off")
end

function B58_LIGHTS_RECOGN_toggle ()
    if logic.And(ipc.readUW("0D0C"),64) == 64 then
		B58_LIGHTS_RECOGN_off ()
	else
		B58_LIGHTS_RECOGN_on ()
	end
end

function B58_LIGHTS_WING_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 128)
    ipc.writeUW("0D0C", n)
	DspShow ("WING", "on")
end

function B58_LIGHTS_WING_off ()
	ipc.clearbitsUW("0D0C", 128)
	DspShow ("WING", "off")
end

function B58_LIGHTS_WING_toggle ()
    if logic.And(ipc.readUW("0D0C"),128) == 128 then
		B58_LIGHTS_WING_off ()
	else
		B58_LIGHTS_WING_on ()
	end
end

function B58_LIGHTS_LOGO_on ()
    n = ipc.readUW("0D0C")
    n = logic.Or(n, 256)
    ipc.writeUW("0D0C", n)
	DspShow ("LOGO", "on")
end

function B58_LIGHTS_LOGO_off ()
	ipc.clearbitsUW("0D0C", 256)
	DspShow ("LOGO", "off")
end

function B58_LIGHTS_LOGO_toggle ()
    if logic.And(ipc.readUW("0D0C"),256) == 256 then
		B58_LIGHTS_LOGO_off ()
	else
		B58_LIGHTS_LOGO_on ()
	end
end

-- ## Electrical Systems ###################################

-- $$ Battery

function B58_BATT_on ()
	ipc.writeDD("281C", 1)
	DspShow ("BAT", "on")
end

function B58_BATT_off ()
	ipc.writeDD("281C", 0)
	DspShow ("BAT", "off")
end

function B58_BATT_toggle ()
    if ipc.readDD("281C") == 0 then
        Battery_on ()
    else
        Battery_off ()
    end
end

-- $$ Avionics Master

function B58_AVIONICS_MASTER_on ()
	ipc.writeUW("2e80", 1)
	DspShow ("AVIO", "on")
end

function B58_AVIONICS_MASTER_off ()
	ipc.writeUW("2e80", 0)
	DspShow ("AVIO", "off ")
end

function B58_AVIONICS_MASTER_toggle ()
    local avio = ipc.readUW("2e80")
    if avio == 1 then
        Avionics_MASTER_off ()
    else
        Avionics_MASTER_on ()
    end
end

-- $$ Alternators
-- uses Generator functions

function B58_ALTNR_1_on ()
    B58_GEN1_on()
	DspShow ("ALT1", "on")
end

function B58_ALTNR_1_off ()
    B58_GEN1_off()
	DspShow ("ALT1", "off")
end

function B58_ALTNR_1_toggle ()
    if ipc.readUW("3B78") == 0 then
        B58_ALTNR_1_on ()
    else
        B58_ALTNR_1_off ()
    end
end

function B58_ALTNR_2_on ()
    B58_GEN2_on()
	DspShow ("ALT2", "on")
end

function B58_ALTNR_2_off ()
    B58_GEN2_off()
	DspShow ("ALT2", "off")
end

function B58_ALTNR_2_toggle ()
    if ipc.readUW("3AB8") == 0 then
        B58_ALTNR_2_on ()
    else
        B58_ALTNR_2_off ()
    end
end

function B58_ALTNR_BOTH_on ()
    B58_GEN1_on()
    B58_GEN2_on()
	DspShow ("ALTs", "on")
end

function B58_ALTNR_BOTH_off ()
    B58_GEN1_off()
    B58_GEN2_off()
	DspShow ("ALTs", "off")
end

function B58_ALTNR_both_toggle ()
    if ipc.readUW("3b78") == 0 then
        B58_ALTNR_BOTH_on()
    else
        B58_ALTNR_BOTH_off ()
    end
end

-- $$ Generators

function B58_GEN1_on ()
	ipc.setbitsUW("3b78", 1)
end

function B58_GEN2_on ()
	ipc.setbitsUW("3ab8", 1)
end

function B58_ALL_GEN_on ()
	ipc.setbitsUW("3b78", 1)
	ipc.setbitsUW("3ab8", 1)
end

function B58_GEN1_off ()
	ipc.clearbitsUW("3b78", 1)
end

function B58_GEN2_off ()
	ipc.clearbitsUW("3ab8", 1)
end

function B58_GEN_ALL_off ()
	ipc.clearbitsUW("3b78", 1)
	ipc.clearbitsUW("3ab8", 1)
end

-- $$ Propeller Sync

function B58_PROPSYNC_toggle ()
	_TOGGLE_PROPELLER_SYNC ()
end

-- $$ Annuniator Test

function B58_ANNUN_test ()
    ipc.writeLvar('ANNUNTEST', 1)
    DspShow('ANNU', 'test')
    ipc.sleep(750)
    ipc.writeLvar('ANNUNTEST', 0)
end

-- ## Fuel System #############################

function B58_FUELPUMPS_ALL_on ()
	ipc.setbitsUW("3b98", 1)
	ipc.setbitsUW("3ad8", 1)
	ipc.setbitsUW("3b98", 1)
	ipc.setbitsUW("3a18", 1)
    DspShow ("PUMP", "on")
end

function B58_FUELPUMPS_ALL_off ()
	ipc.clearbitsUW("3b98", 1)
	ipc.clearbitsUW("3ad8", 1)
	ipc.clearbitsUW("3b98", 1)
	ipc.clearbitsUW("3a18", 1)
    DspShow ("PUMP", "off")
end

function B58_FUELPUMPS_ALL_toggle ()
	if ipc.readUB(0x3104) == 0 then
        B58_FUELPUMPS_ALL_on ()
	else
        B58_FUELPUMPS_ALL_off ()
	end
end

function B58_FUELSEL_ALL_on ()
	ipc.setbitsUW("3880", 1)
	ipc.setbitsUW("37c0", 1)
	ipc.setbitsUW("3700", 1)
	ipc.setbitsUW("3640", 1)
end

function B58_FUELSEL_ALL_off ()
	ipc.clearbitsUW("3880", 1)
	ipc.clearbitsUW("37c0", 1)
	ipc.clearbitsUW("3700", 1)
	ipc.clearbitsUW("3640", 1)
end

-- ## Landing Gear ##########################

function B58_GEAR_up ()
	_GEAR_UP ()
	DspShow ("GEAR", "up")
end

function B58_GEAR_down ()
	_GEAR_DOWN ()
	DspShow ("GEAR", "down")
end

function B58_GEAR_toggle ()
	_GEAR_TOGGLE ()
end

function B58_BRAKES ()
	_BRAKES ()
end

function B58_BRAKES_PARKING_toggle ()
	_PARKING_BRAKES ()
end

-- ## DeIce ###################################

-- $$ Pitot Deice

function B58_DEICE_PITOT_on ()
	_PITOT_HEAT_ON ()
end

function B58_DEICE_PITOT_off ()
	_PITOT_HEAT_OFF ()
end

function B58_DEICE_PITOT_toggle ()
	_PITOT_HEAT_TOGGLE ()
end

-- $$ Propeller Deice

function B58_DEICE_PROP_on ()
	ipc.setbitsUW("2440", 1)
end

function B58_DEICE_PROP_off ()
	ipc.clearbitsUW("2440", 1)
end

function B58_DEICE_PROP_toggle ()
	_TOGGLE_PROPELLER_DEICE ()
end

-- $$ Fuel Deice

function B58_DEICE_FUEL_on ()
    if ipc.readLvar('Cabinlever13') ~= 1 then
        ipc.writeLvar('Cabinlever13', 1)
    end
end

function B58_DEICE_FUEL_off ()
    if ipc.readLvar('Cabinlever13') ~= 0 then
        ipc.writeLvar('Cabinlever13', 0)
    end
end

function B58_DEICE_FUEL_toggle ()
    if ipc.readLvar('Cabinlever13') ~= 1 then
        B58_DEICE_FUEL_on()
    else
        B58_DEICE_FUEL_off()
    end
end

-- $$ Stall Deice

function B58_DEICE_STALL_on ()
    if ipc.readLvar('Cabinlever12') ~= 1 then
        ipc.writeLvar('Cabinlever12', 1)
    end
end

function B58_DEICE_STALL_off ()
    if ipc.readLvar('Cabinlever12') ~= 0 then
        ipc.writeLvar('Cabinlever12', 0)
    end
end

function B58_DEICE_STALL_toggle ()
    if ipc.readLvar('Cabinlever12') ~= 1 then
        B58_DEICE_STALL_on()
    else
        B58_DEICE_STALL_off()
    end
end

-- $$ Windscreen Deice

function B58_DEICE_WIND_on ()
    if ipc.readLvar('Cabinlever4') ~= 1 then
        ipc.writeLvar('Cabinlever4', 1)
    end
end

function B58_DEICE_WIND_off ()
    if ipc.readLvar('Cabinlever4') ~= 0 then
        ipc.writeLvar('Cabinlever4', 0)
    end
end

function B58_DEICE_WIND_toggle ()
    if ipc.readLvar('Cabinlever4') ~= 1 then
        B58_DEICE_WIND_on()
    else
        B58_DEICE_WIND_off()
    end
end

-- $$ Carbarettor Heater

function B58_CARBHEAT_on ()
	ipc.setbitsUW("08b2", 1)
end

function B58_CARBHEAT_off ()
	ipc.clearbitsUW("08b2", 1)
end

-- $$ All Deice

function B58_DEICE_ALL_on ()
    B58_DEICE_FUEL_on ()
    B58_DEICE_STALL_on ()
    B58_DEICE_PROP_on ()
    B58_DEICE_WIND_on ()
end

function B58_DEICE_ALL_off ()
    B58_DEICE_FUEL_off ()
    B58_DEICE_STALL_off ()
    B58_DEICE_PROP_off ()
    B58_DEICE_WIND_off ()
end

function B58_DEICE_ALL_toggle ()
    if ipc.readLvar('Cabinlever4') ~= 1 then
        B58_DEICE_ALL_on()
    else
        B58_DEICE_ALL_off()
    end
end

-- ## Flight controls ############

function B58_FLAPS_up ()
	_FLAPS_UP ()
end

function B58_FLAPS_down ()
	_FLAPS_DOWN ()
end

function B58_FLAPS_incr ()
-- flaps inc operates in reverse sense
	_FLAPS_DECR ()
end

function B58_FLAPS_decr ()
-- flaps dec operates in reverse sense
	_FLAPS_INCR ()
end

-- ## ATC window ############

function ATC_WINDOW_toggle ()
	_ATC ()
end

function ATC_MENU_0 ()
	_ATC_MENU_0 ()
end

function ATC_MENU_1 ()
	_ATC_MENU_1 ()
end

function ATC_MENU_2 ()
	_ATC_MENU_2 ()
end

function ATC_MENU_3 ()
	_ATC_MENU_3 ()
end

function ATC_MENU_4 ()
	_ATC_MENU_4 ()
end

function ATC_MENU_5 ()
	_ATC_MENU_5 ()
end

function ATC_MENU_6 ()
	_ATC_MENU_6 ()
end

function ATC_MENU_7 ()
	_ATC_MENU_7 ()
end

function ATC_MENU_8 ()
	_ATC_MENU_8 ()
end

function ATC_MENU_9 ()
	_ATC_MENU_9 ()
end

-- ## Engines starting

function B58_ENG_start ()
	_ENGINE_AUTO_START ()
end

function B58_ENG_stop ()
	_ENGINE_AUTO_SHUTDOWN ()
end

function B58_ENG_ALL_start ()
    B58_ENG1_start()
    B58_ENG2_start()
end

function B58_ENG1_start ()
	val = ipc.readSB("0892")
	if val == 3 then
    	_MAGNETO1_START ()
	end
end

function B58_ENG2_start ()
	val = ipc.readSB("092A")
	if val == 3 then
    	_MAGNETO2_START ()
	end
end

----------- Magnetos toggle just from Off to BOTH

function B58_ENG_MAG_show ()
    if FVar == "0892" then
        EngMV = "1"
    elseif FVar == "092A" then
        EngMV = "2"
    elseif FVar == "09C2" then
        EngMV = "3"
    elseif FVar == "0A5A" then
        EngMV = "4"
    end
    if val == 0 then
        MagPos = " off"
    elseif val == 1 then
        MagPos = "rght"
    elseif val == 2 then
        MagPos = "left"
    elseif val == 3 then
        MagPos = "both"
    end
    DspShow("MAG" .. EngMV, MagPos)
end

function B58_ENG_ALL_MAG_R ()
    B58_ENG1_MAG_R()
    B58_ENG2_MAG_R()
end

function B58_ENG_ALL_MAG_L ()
	B58_ENG1_MAG_L()
    B58_ENG2_MAG_L()
end

function B58_ENG_ALL_MAG_both ()
    B58_ENG1_MAG_both()
    B58_ENG2_MAG_both()
end

function B58_ENG_ALL_MAG_inc ()
    B58_ENG1_MAG_inc()
    B58_ENG2_MAG_inc()
end

function B58_ENG_ALL_MAG_dec ()
    B58_ENG1_MAG_dec()
    B58_ENG2_MAG_dec()
end

function B58_ENG1_MAG_R ()
	FVar = "0892"
    val = 1
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG1_MAG_L ()
	FVar = "0892"
    val = 2
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG1_MAG_both ()
	FVar = "0892"
    val = 3
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG1_MAG_inc ()
	FVar = "0892"
	val = ipc.readSB(FVar)
	if val <= 2 then
    	val = val + 1
	end
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG1_MAG_dec ()
	FVar = "0892"
	val = ipc.readSB(FVar)
	if val >= 1 then
    	val = val - 1
	end
 	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG2_MAG_R ()
	FVar = "092A"
    val = 1
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG2_MAG_L ()
	FVar = "092A"
    val = 2
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG2_MAG_both ()
	FVar = "092A"
    val = 3
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG2_MAG_inc ()
	FVar = "092A"
	val = ipc.readSB(FVar)
	if val <= 2 then
    	val = val + 1
	end
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

function B58_ENG2_MAG_dec ()
	FVar = "092A"
	val = ipc.readSB(FVar)
	if val >= 1 then
    	val = val - 1
	end
	ipc.writeSB(FVar, val)
    B58_ENG_MAG_show ()
end

-- ## Trims ###############

-- $$ Evevator Trim

function B58_ELEV_Trim_show ()
    ElevVal = ipc.readUW(0x0BC2)
    if ElevVal >= 49000 then
        ElevVal = (ElevVal - 65487)
    end
    ElevTrimVar = round2(ElevVal*0.00091552734375,2)
    DspShow("ELEV", ElevTrimVar)
end

function B58_ELEV_Trim_up ()
    ipc.control(65615)
    Elevator_Trim_show ()
end

function B58_ELEV_Trim_dn ()
    ipc.control(65607)
    Elevator_Trim_show ()
end

function B58_ELEV_Trim_upfast ()
    i = 0
    while i <= 4 do
        ipc.control(65615)
        i = i + 1
    end
    Elevator_Trim_show ()
end

function B58_ELEV_Trim_dnfast ()
    i = 0
    while i <= 4 do
        ipc.control(65607)
        i = i + 1
    end
    Elevator_Trim_show ()
end

-- $$ Aileron Trim

function B58_AIL_Trim_show ()
    AilVal = ipc.readUW(0x0C02)
    if AilVal >= 49000 then
        AilVal = (AilVal - 65487)
    end
    AilTrimVar = round(AilVal*0.006103515625)
    DspShow("AIL", AilTrimVar)
end

function B58_AIL_Trim_left ()
    ipc.control(66276)
    Aileron_Trim_show ()
end

function B58_AIL_Trim_right ()
    ipc.control(66277)
    Aileron_Trim_show ()
end

function B58_AIL_Trim_reset ()
    ipc.writeSB("0C02", 0)
    ipc.sleep(50)
    Aileron_Trim_show ()
end

-- $$ Rudder Trim

function B58_RUD_Trim_show ()
    RuddVal = ipc.readUW(0x0C04)
    if RuddVal >= 49000 then
    RuddVal = (RuddVal - 65487)
    end
    RuddTrimVar = math.ceil(RuddVal*0.006103515625)
    DspShow("RUDD", RuddTrimVar)
end

function B58_RUD_Trim_left ()
    ipc.control(66278)
    Rudder_Trim_show ()
end

function B58_RUD_Trim_right ()
    ipc.control(66279)
    Rudder_Trim_show ()
end

function B58_RUD_Trim_reset ()
    ipc.writeSB("0C04", 0)
    ipc.sleep(50)
    Rudder_Trim_show ()
end

-- ## Doors ###############

function B58_DOOR_MAIN_open ()
    if logic.And(ipc.readUW("3367"), 1) ~= 1 then
        ipc.setbitsUW("3367", 1)
        DspShow ("MAIN", "open")
        ipc.sleep(1000)
    end
end

function B58_DOOR_MAIN_close ()
    if logic.And(ipc.readUW("3367"), 1) ~= 0 then
        ipc.clearbitsUW("3367", 1)
        DspShow ("MAIN", "clse")
        ipc.sleep(1000)
    end
end

function B58_DOOR_MAIN_toggle ()
    if logic.And(ipc.readUW("3367"), 1) == 1 then
		B58_DOOR_MAIN_close ()
	else
		B58_DOOR_MAIN_open ()
    end
end

function B58_DOOR_PASS_open ()
    if logic.And(ipc.readUW("3367"), 2) ~= 2 then
        ipc.setbitsUW("3367", 2)
        DspShow ("PASS", "open")
        ipc.sleep(1000)
    end
end

function B58_DOOR_PASS_close ()
    if logic.And(ipc.readUW("3367"), 2) ~= 0 then
        ipc.clearbitsUW("3367", 2)
        DspShow ("PASS", "clse")
        ipc.sleep(1000)
    end
end

function B58_DOOR_PASS_toggle ()
    if logic.And(ipc.readUW("3367"), 2) == 2 then
		B58_DOOR_PASS_close ()
	else
		B58_DOOR_PASS_open ()
    end
end

function B58_DOOR_BAGS_open ()
    if logic.And(ipc.readUW("3367"), 4) ~= 4 then
        ipc.setbitsUW("3367", 4)
        DspShow ("BAGS", "open")
        ipc.sleep(1000)
    end
end

function B58_DOOR_BAGS_close ()
    if logic.And(ipc.readUW("3367"), 4) ~= 0 then
        ipc.clearbitsUW("3367", 4)
        DspShow ("BAGS", "clse")
        ipc.sleep(1000)
    end
end

function B58_DOOR_BAGS_toggle ()
    if logic.And(ipc.readUW("3367"), 4) == 4 then
		B58_DOOR_BAGS_close ()
	else
		B58_DOOR_BAGS_open ()
    end
end

function B58_DOOR_ALL_open ()
    -- open door when loading
    OnGround = ipc.readUW("0366") -- plane on ground?
    Eng1Run = ipc.readUW("0894") -- Engine 1 running?
    Eng2Run = ipc.readUW("092C") -- Engine 2 running?
    EngSumVar = Eng1Run + Eng2Run
    -- if on ground and engines off, open door
    if OnGround == 1 and EngSumVar == 0 then
        B58_DOOR_MAIN_open()
        ipc.sleep(3000)
        B58_DOOR_PASS_open()
    end
end

function B58_DOOR_ALL_close()
    B58_DOOR_PASS_close()
    ipc.sleep(3000)
    B58_DOOR_MAIN_close()
    ipc.sleep(3000)
    B58_DOOR_BAGS_close()
end

function B58_DOOR_ALL_toggle ()
    if ipc.readUW("3367") ~= 0 then
        B58_DOOR_ALL_close()
    else
        B58_DOOR_ALL_open()
    end
end

function B58_DOOR_PILOT_open ()
    if ipc.readLvar('Cabinlever100') ~= 1 then
        ipc.writeLvar('Cabinlever100', 1)
        DspShow('WIND','open')
    end
end

function B58_DOOR_PILOT_close ()
    if ipc.readLvar('Cabinlever100') ~= 0 then
        ipc.writeLvar('Cabinlever100', 0)
        DspShow('WIND','clse')
    end
end

function B58_DOOR_PILOT_toggle ()
    if ipc.readLvar('Cabinlever100') ~= 1 then
        B58_DOOR_PILOT_open()
    else
        B58_DOOR_PILOT_close()
    end
end

-- ## Cabin Conditioning

-- $$ Cabin Heater

function B58_COND_HEATER_on ()
    if ipc.readLvar('Cabinlever1') ~= 1 then
        ipc.writeLvar('Cabinlever1', 1)
        DspShow('HEAT','on')
    end
end

function B58_COND_HEATER_off ()
    if ipc.readLvar('Cabinlever1') ~= 0 then
        ipc.writeLvar('Cabinlever1', 0)
        DspShow('HEAT','off')
    end
end

function B58_COND_HEATER_toggle ()
    if ipc.readLvar('Cabinlever1') ~= 1 then
        B58_COND_HEATER_on()
    else
        B58_COND_HEATER_off()
    end
end

-- $$ Cabin Blower

function B58_COND_BLOWER_on ()
    if ipc.readLvar('Cabinlever2') ~= 1 then
        ipc.writeLvar('Cabinlever2', 1)
        DspShow('BLOW','on')
    end
end

function B58_COND_BLOWER_off ()
    if ipc.readLvar('Cabinlever2') ~= 0 then
        ipc.writeLvar('Cabinlever2', 0)
        DspShow('BLOW','off')
    end
end

function B58_COND_BLOWER_toggle ()
    if ipc.readLvar('Cabinlever2') ~= 1 then
        B58_COND_BLOWER_on()
    else
        B58_COND_BLOWER_off()
    end
end

-- $$ Cabin Heat

function B58_COND_CABIN_HEAT_on ()
    if ipc.readLvar('Cabinlever8') ~= 1 then
        ipc.writeLvar('Cabinlever8', 1)
        DspShow('CBHT','on')
    end
end

function B58_COND_CABIN_HEAT_off ()
    if ipc.readLvar('Cabinlever8') ~= 0 then
        ipc.writeLvar('Cabinlever8', 0)
        DspShow('CBHT','off')
    end
end

function B58_COND_CABIN_HEAT_toggle ()
    if ipc.readLvar('Cabinlever8') ~= 1 then
        B58_COND_CABIN_HEAT_on()
    else
        B58_COND_CABIN_HEAT_off()
    end
end

-- $$ Aft Cabin Heat

function B58_COND_PASS_HEAT_on ()
    if ipc.readLvar('Cabinlever9') ~= 1 then
        ipc.writeLvar('Cabinlever9', 1)
        DspShow('PSHT','on')
    end
end

function B58_COND_PASS_HEAT_off ()
    if ipc.readLvar('Cabinlever9') ~= 0 then
        ipc.writeLvar('Cabinlever9', 0)
        DspShow('PAHT','off')
    end
end

function B58_COND_PASS_HEAT_toggle ()
    if ipc.readLvar('Cabinlever9') ~= 1 then
        B58_COND_PASS_HEAT_on()
    else
        B58_COND_PASS_HEAT_off()
    end
end

-- $$ Cabin Heat

function B58_COND_DEFROST_on ()
    if ipc.readLvar('Cabinlever10') ~= 1 then
        ipc.writeLvar('Cabinlever10', 1)
        DspShow('DFST','on')
    end
end

function B58_COND_DEFROST_off ()
    if ipc.readLvar('Cabinlever10') ~= 0 then
        ipc.writeLvar('Cabinlever10', 0)
        DspShow('DFST','off')
    end
end

function B58_COND_DEFROST_toggle ()
    if ipc.readLvar('Cabinlever10') ~= 1 then
        B58_COND_DEFROST_on()
    else
        B58_COND_DEFROST_off()
    end
end

-- $$ All Heater Controls

function B58_COND_ALL_on ()
    B58_COND_CABIN_HEAT_on()
    B58_COND_PASS_HEAT_on()
    B58_COND_DEFROST_on()
end

function B58_COND_ALL_off ()
    B58_COND_CABIN_HEAT_off()
    B58_COND_PASS_HEAT_off()
    B58_COND_DEFROST_off()
end

function B58_COND_ALL_toggle()
    B58_COND_CABIN_HEAT_toggle()
    B58_COND_PASS_HEAT_toggle()
    B58_COND_DEFROST_toggle()
end

-- ## Save Flights ####################

function B58_Save_Current_Flight()
    SaveFlight()
end

-- ## Other ###########################

-- $$ Yokes

function B58_YOKES_hide ()
    ipc.writeLvar('YokePilot', 1)
    ipc.writeLvar('YokeCoPilot', 1)
    DspShow('YOKE', 'hide')
end

function B58_YOKES_show ()
    ipc.writeLvar('YokePilot', 0)
    ipc.writeLvar('YokeCoPilot', 0)
    DspShow('YOKE', 'show')
end

function B58_YOKES_toggle ()
    if ipc.readLvar('YokePilot') == 1 then
        B58_YOKES_show()
    else
        B58_YOKES_hide()
    end
end

-- $$ Sunvisors

function B58_SUNVISOR_L_down ()
    if ipc.readLvar('Sunvisor1') ~= 1 then
        ipc.writeLvar('Sunvisor1', 1)
        DspShow('SUNL', 'down')
    end
end

function B58_SUNVISOR_L_up ()
    if ipc.readLvar('Sunvisor1') ~= 0 then
        ipc.writeLvar('Sunvisor1', 0)
        DspShow('SUNL', 'up')
    end
end

function B58_SUNVISOR_L_toggle ()
    if ipc.readLvar('Sunvisor1') ~= 0 then
        B58_SUNVISOR_L_up()
    else
        B58_SUNVISOR_L_down()
    end
end

function B58_SUNVISOR_R_down ()
    if ipc.readLvar('Sunvisor2') ~= 1 then
        ipc.writeLvar('Sunvisor2', 1)
        DspShow('SUNR', 'down')
    end
end

function B58_SUNVISOR_R_up ()
    if ipc.readLvar('Sunvisor2') ~= 0 then
        ipc.writeLvar('Sunvisor2', 0)
        DspShow('SUNR', 'up')
    end
end

function B58_SUNVISOR_R_toggle ()
    if ipc.readLvar('Sunvisor2') ~= 0 then
        B58_SUNVISOR_R_up()
    else
        B58_SUNVISOR_R_down()
    end
end

function B58_SUNVISOR_LR_down ()
    B58_SUNVISOR_L_down ()
    B58_SUNVISOR_R_down ()
end

function B58_SUNVISOR_LR_up ()
    B58_SUNVISOR_L_up ()
    B58_SUNVISOR_R_up ()
end

function B58_SUNVISOR_LR_toggle ()
    if ipc.readLvar('Sunvisor1') ~= 0 then
        B58_SUNVISOR_LR_up()
    else
        B58_SUNVISOR_LR_down()
    end
end

-- $$ Chronometer

function B58_CHRON_select ()
    if ipc.readLvar('Modo') ~= 1 then
        ipc.writeLvar('Modo', 1)
    else
        ipc.writeLvar('Modo', 0)
    end
end

-- not working - unable to reset stopwatch
function B58_CHRON_control ()
    ipc.writeLvar('Trans', ipc.readLvar('Trans'))
    --ipc.writeLvar('VSMODETIME', 1)
    ipc.writeLvar('Chron', 0)
    ipc.writeLvar('BotonCTL', 0)
    ipc.sleep(1500)
    ipc.writeLvar('Chron', 1)
    ipc.writeLvar('BotonCTL', 1)
    --ipc.writeLvar('VSMODETIME', 0)
    --[[
    if ipc.readLvar('VSMODETIME') ~= 1 then
        ipc.writeLvar('Chron', 0)
        ipc.sleep(200)
        ipc.writeLvar('Time1', 0)
        ipc.writeLvar('SecondsTime2', 0)
        ipc.writeLvar('VSMODETIME', 0)
        ipc.sleep(200)
        ipc.writeLvar('Chron', 1)
    else
        ipc.sleep(200)
        ipc.writeLvar('Chron', 1)
    end
    --]]
end

function B58_Set_UTC ()
    simHour = ipc.readUB("023B")
    simMinute = ipc.readUB("023c")
    Hour = os.date("!%H")
    Minute = os.date("!%M")
    questvar = ipc.ask("Setting current sim time (".. simHour..":"..simMinute..") to UTC (".. Hour..":"..Minute..")? [y / n]")
    if questvar == "y" then
        Hour = os.date("!%H")
        Minute = os.date("!%M")
        ipc.writeUB("023B", Hour)
        ipc.writeUB("023c", Minute)
    end
end

-- ## System functions ##

-- Initial info on MCP display
function InitDsp ()
    DspCRS(CRS1)
    SPD = ipc.readUW(0x07E2)
	DspSPD(SPD)
	ipc.sleep(40)
    HDG = round(ipc.readUW(0x07CC)*360/65536)
	DspHDG(HDG)
	ipc.sleep(40) -- << waiting for FSX data update
    ALT = round(ipc.readUD(0x07D4)/65536*3.28084/100)
	DspALT(ALT)
	DspClear()
end

-- Initial variables
function FallbackInitVars ()
	-- Tuning fast rotation speeds
	fast_alt = 10 -- << increment value when rotary in fast mode
	fast_hdg = 10
	fast_crs = 5
	fast_ias = 10
	fast_baro = 10
    fast_spd = 10
	-- init CRS direction
    crs_hdg = ipc.readUW(0x0C4E)
    if _MCP1 () then
        -- don't need this on MCP2
	    ipc.writeUW(0x0C4E, 0)
    end
	-- Setting initial varibles values
	prev_mask = 0
	prev_num = 0
	tcas_mode = 0
	ipc.writeUW("0D0C", 0)
    AUTO_SAVE_ENABLED = true
end

function Timer ()
    AdditionalInfo ()
end

function AdditionalInfo ()
    local flappos = ipc.readUW('0BE0')
    local flapsel = ipc.readUW('0BDC')

    local gearsel = ipc.readUW('0BE8')
    local gearnose = ipc.readUW('0BEC')
    local gearleft = ipc.readUW('0BF0')
    local gearright = ipc.readUW('0BF4')
    local gearpos = gearnose + gearleft + gearright
    local ias = math.floor(ipc.readUW("02BC")/128)
    local overspeed = ''
    local ap = 'ap'
    local tx = ipc.readLvar("xpdr_onoff_knob_pos")

    -- flaps selection and position
    if flapsel == 0 then
        flaps = 'UP'
    elseif flapsel == 8192 then
        flaps = 'AP'
    elseif flapsel == 16383 then
        flaps = 'DN'
    end
    if flapsel < flappos then
        flaps = flaps .. '<'
    elseif flapsel > flappos then
        flaps = flaps .. '>'
    end

    -- landing gear
    if gearsel == 0 then
        gear = 'UP'
    else
        gear = 'DN'
    end
    if gearsel == 0 and (gearpos > 0) then
        gear = gear .. '<'
    elseif gearsel == 16383 and (gearpos < 49149) then
        gear = gear .. '>'
    end

    -- autopilot state
    if ipc.readUD(0x07BC) == 1 then
        ap = 'AP'
    end

    -- Flap restrictions
    if ias > 152 and flappos > 0 then
        overspeed = '!'
    end

    if ias > 122 and flappos > 8192 then
        overspeed = '!'
    end

    if not _MCP1() then
        FLIGHT_INFO1 = ap .. " G:" .. gear
        FLIGHT_INFO2 = '  ' .. " F:" .. flaps .. overspeed
    else
        FLIGHT_INFO1 = "G:" .. gear
        FLIGHT_INFO2 = "F:".. flaps .. overspeed
        -- second line is left for AP indication
    end
    DisplayResync ()
end

function InitVars ()
    -- uncomment to disable display
    -- AutopilotDisplayBlocked ()

    AutoDisplay = false -- disable default AP knob functions
    SPD_CRS_replace = true -- replace SPD indication with CRS indication for bush planes

    temp_units = 0 -- 0 - C, 1 - F

    if RADIOS ~= nil then -- prevent error on joysticks-only setup
        RADIOS["ADF1 Swap"]	=  A172_ADFbuttonFrq_click
        RADIOS["ADF ."]	=  A172_ADF_select_x
    end
end

